package ksoapTests;
 
import java.io.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
 
import org.ksoap.*;
import org.ksoap.marshal.*;
import org.kobjects.serialization.*;
import org.kobjects.isodate.*;
import org.kxml.*;
import org.kxml.io.*;
import org.kxml.parser.*;
 
public class StockOrderResponse extends MIDlet implements CommandListener {
 
  private Display display;
  private Command doneCommand;
  private TextField textField;
 
  public StockOrderResponse () throws Exception {
    // Initialize display and command objects.
    display = Display.getDisplay(this);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
  }

  // A RPC response style SOAP Stock Order response message.
  private String soapRespMesg =
  "<SOAP-ENV:Envelope " +
    "xmlns:SOAP-ENV=\"http://www.w3.org/2001/12/soap-envelope\" " +
    "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
    "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">" +
       "<SOAP-ENV:Body>" +
          "<result>" +
            "<OrderStatus>" +
              "<CustomerName xsi:type=\"xsd:string\">Michael Yuan</CustomerName>" +
              "<Symbol xsi:type=\"xsd:string\">XYZ</Symbol>" +
              "<Share xsi:type=\"xsd:int\">1000</Share>" +
              "<Buy xsi:type=\"xsd:boolean\">true</Buy>" +
              "<Price xsi:type=\"xsd:float\">123.45</Price>" +
              "<ExecTime xsi:type=\"xsd:dateTime\">2002-07-18T23:20:50.52Z</ExecTime>" +
            "</OrderStatus>" +
          "</result>" +
       "</SOAP-ENV:Body>" +
  "</SOAP-ENV:Envelope>";

  // A RPC response style SOAP Stock Order response message.
  private String complexSoapRespMesg =
  "<SOAP-ENV:Envelope " +
    "xmlns:SOAP-ENV=\"http://www.w3.org/2001/12/soap-envelope\" " +
    "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
    "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" " +
    "xmlns:n=\"http://www.javaworld.com/ksoap/test\">" +
       "<SOAP-ENV:Body>" +
          "<result>" +
            "<OrderStatus xsi:type=\"n:orderStatus\">" +
              "<CustomerName xsi:type=\"xsd:string\">Michael Yuan</CustomerName>" +
              "<Transaction xsi:type=\"n:transaction\">" +
                "<Symbol xsi:type=\"xsd:string\">XYZ</Symbol>" +
                "<Share xsi:type=\"xsd:int\">1000</Share>" +
                "<Buy xsi:type=\"xsd:boolean\">true</Buy>" +
                "<Price xsi:type=\"xsd:float\">123.45</Price>" +
              "</Transaction>" +
              "<ExecTime xsi:type=\"xsd:dateTime\">2002-07-18T23:20:50.52Z</ExecTime>" +
            "</OrderStatus>" +
          "</result>" +
       "</SOAP-ENV:Body>" +
  "</SOAP-ENV:Envelope>";

  // A RPC response style SOAP Stock Order response message.
  private String arraySoapRespMesg =
  "<SOAP-ENV:Envelope " +
    "xmlns:SOAP-ENV=\"http://www.w3.org/2001/12/soap-envelope\" " +
    "xmlns:SOAP-ENC=\"http://www.w3.org/2001/12/soap-encoding\" " +
    "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
    "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" " +
    "xmlns:n=\"http://www.javaworld.com/ksoap/test\">" +
       "<SOAP-ENV:Body>" +
          "<result>" +
            "<OrderStatus xsi:type=\"n:orderStatus\">" +
              "<CustomerName xsi:type=\"xsd:string\">Michael Yuan</CustomerName>" +
              "<Transactions xsi:type=\"SOAP-ENC:Array\" SOAP-ENC:arrayType=\"n:transaction[2]\">" +
                "<Transaction xsi:type=\"n:transaction\">" +
                  "<Symbol xsi:type=\"xsd:string\">ABC</Symbol>" +
                  "<Share xsi:type=\"xsd:int\">500</Share>" +
                  "<Buy xsi:type=\"xsd:boolean\">true</Buy>" +
                  "<Price xsi:type=\"xsd:float\">43.21</Price>" +
                "</Transaction>" +
                "<Transaction xsi:type=\"n:transaction\">" +
                  "<Symbol xsi:type=\"xsd:string\">XYZ</Symbol>" +
                  "<Share xsi:type=\"xsd:int\">1000</Share>" +
                  "<Buy xsi:type=\"xsd:boolean\">true</Buy>" +
                  "<Price xsi:type=\"xsd:float\">123.45</Price>" +
                "</Transaction>" +
              "</Transactions>" +
              "<ExecTime xsi:type=\"xsd:dateTime\">2002-07-18T23:20:52.52Z</ExecTime>" +
            "</OrderStatus>" +
          "</result>" +
       "</SOAP-ENV:Body>" +
  "</SOAP-ENV:Envelope>";

  private SoapObject processRespMesg01 (String mesg) throws Exception {
    ByteArrayInputStream bis = new ByteArrayInputStream (mesg.getBytes ());
    InputStreamReader reader = new InputStreamReader (bis);
    XmlParser xp = new XmlParser (reader);
    // Use default mapping between Java objects and Soap elements;
    SoapEnvelope envelope = new SoapEnvelope (new ClassMap (Soap.VER12));
    envelope.parse (xp);

    // Get the first grandchild under "Body" element.
    //
    // This is to take into account that "Result" element
    // is often the direct child of "Body" and any real information
    // is encapsulated under the child of "Result".
    // 
    // Since the grandchild of "Body" is "OrderStatus" which is
    // not a simple type, composite "SoapObject" is used here.
    return (SoapObject) envelope.getResult();
  }

  private SoapObject processRespMesg02 (String mesg) throws Exception {
    ByteArrayInputStream bis = new ByteArrayInputStream (mesg.getBytes ());
    InputStreamReader reader = new InputStreamReader (bis);
    XmlParser xp = new XmlParser (reader);

    // Register Marshal for "xsd:dateTime" type.
    ClassMap cm = new ClassMap (Soap.VER12);
    Marshal md = new MarshalDate ();
    md.register (cm);
    SoapEnvelope envelope = new SoapEnvelope (cm);
    envelope.parse (xp);
 
    return (SoapObject) envelope.getResult();
  }

  private SoapObject processRespMesg03 (String mesg) throws Exception {
    ByteArrayInputStream bis = new ByteArrayInputStream (mesg.getBytes ());
    InputStreamReader reader = new InputStreamReader (bis);
    XmlParser xp = new XmlParser (reader);

    ClassMap cm = new ClassMap (Soap.VER12);

    // Register Marshal for "xsd:dateTime" type.
    Marshal md = new MarshalDate ();
    md.register (cm);

    // Add a "SoapObject" template for "Transaction" element.
    // 
    // If the template contains less properties (elements) than 
    // the real SOAP message elements, the parser throws an
    // exception. The exception is "Unknown Property" indicating
    // the missing property.
    //
    // There will be no error, however, if the template contains
    // more element than the real SOAP message or if the value object 
    // types in the template is wrong. In fact, we can use "null" for
    // all the values in the template.
    SoapObject so = new SoapObject ("http://www.javaworld.com/ksoap/test", 
                                    "transaction");
    so.addProperty ("Symbol", new String (""));
    so.addProperty ("Share", new Integer (0));
    so.addProperty ("Buy", new Boolean (true));
    so.addProperty ("Price", new SoapPrimitive ("xsd", "float", ""));
    cm.addTemplate (so);

    SoapEnvelope envelope = new SoapEnvelope (cm);
    envelope.parse (xp);
 
    return (SoapObject) envelope.getResult();
  }

  public void startApp() {

    Form form = new Form ("Stock Order Response");

    // Example #1 Pass #1
    try {
      SoapObject orderStatus = processRespMesg01 (soapRespMesg);

      String customerName = (String) orderStatus.getProperty ("CustomerName");
      String symbol = (String) orderStatus.getProperty ("Symbol");
      Integer share = (Integer) orderStatus.getProperty ("Share");
      Boolean buy = (Boolean) orderStatus.getProperty ("Buy");
      // Since MIDP has no "Float" type, there is no corresponding
      // Java object type for "xsd:float". So, as any unknown type,
      // this element is mapped to a "SoapPrimitive".
      SoapPrimitive price = (SoapPrimitive) orderStatus.getProperty ("Price");
      SoapPrimitive execTime = (SoapPrimitive) orderStatus.getProperty ("ExecTime");

      form.append (new StringItem ("Example #1 Pass #1", ""));
      form.append (new StringItem ("Customer Name", customerName));
      form.append (new StringItem ("Stock Symbol", symbol));
      form.append (new StringItem ("Share", share.toString()));
      form.append (new StringItem ("Buy?", buy.toString()));
      // The "toString()" method of a "SoapPrimitive" object
      // returns the string value of the XML text used to construct
      // this object. Methods "getNamespace()" and "getName()" will 
      // return "xsd:float" and "isodate" for these two objects.
      form.append (new StringItem ("Price", price.toString()));
      form.append (new StringItem ("ExecTime", execTime.toString()));
    } catch (Exception e) {
      e.printStackTrace ();
    }

    // Example #1 Pass #2
    try {
      SoapObject orderStatus = processRespMesg02 (soapRespMesg);

      String customerName = (String) orderStatus.getProperty ("CustomerName");
      String symbol = (String) orderStatus.getProperty ("Symbol");
      Integer share = (Integer) orderStatus.getProperty ("Share");
      Boolean buy = (Boolean) orderStatus.getProperty ("Buy");
      SoapPrimitive price = (SoapPrimitive) orderStatus.getProperty ("Price");
      // Element "ExecTime" was marshaled into an "Date"
      // Java type object.
      Date execTime = (Date) orderStatus.getProperty ("ExecTime");
      Calendar cal = Calendar.getInstance();
      cal.setTime (execTime);

      form.append (new StringItem ("Example #1 Pass #2", ""));
      form.append (new StringItem ("Customer Name", customerName));
      form.append (new StringItem ("Stock Symbol", symbol));
      form.append (new StringItem ("Share", share.toString()));
      form.append (new StringItem ("Buy?", buy.toString()));
      form.append (new StringItem ("Price", price.toString()));
      form.append (new StringItem ("ExecTime", 
                     cal.get(Calendar.HOUR_OF_DAY) + ":" +
                     cal.get(Calendar.MINUTE) + ":" +
                     cal.get(Calendar.SECOND) + " " +
                     cal.get(Calendar.YEAR) + "-" +
                     cal.get(Calendar.MONTH) + "-" +
                     cal.get(Calendar.DATE)
                  ));
    } catch (Exception e) {
      e.printStackTrace ();
    }

    // Example #2 Pass #1
    try {
      SoapObject orderStatus = processRespMesg01 (complexSoapRespMesg);

      String customerName = (String) orderStatus.getProperty ("CustomerName");

      SoapObject transaction = (SoapObject) orderStatus.getProperty ("Transaction");
      String symbol = (String) transaction.getProperty ("Symbol");
      Integer share = (Integer) transaction.getProperty ("Share");
      Boolean buy = (Boolean) transaction.getProperty ("Buy");
      SoapPrimitive price = (SoapPrimitive) transaction.getProperty ("Price");

      SoapPrimitive execTime = (SoapPrimitive) orderStatus.getProperty ("ExecTime");

      form.append (new StringItem ("Example #2 Pass #1", ""));
      form.append (new StringItem ("Customer Name", customerName));
      form.append (new StringItem ("Stock Symbol", symbol));
      form.append (new StringItem ("Share", share.toString()));
      form.append (new StringItem ("Buy?", buy.toString()));
      form.append (new StringItem ("Price", price.toString()));
      form.append (new StringItem ("ExecTime", execTime.toString()));
    } catch (Exception e) {
      e.printStackTrace ();
    }

    // Example #2 Pass #2
    try {
      SoapObject orderStatus = processRespMesg03 (complexSoapRespMesg);
// System.out.println ("OrderStatus:" + orderStatus.getNamespace () +
//                     ":" + orderStatus.getName ());

      String customerName = (String) orderStatus.getProperty ("CustomerName");

      SoapObject transaction = (SoapObject) orderStatus.getProperty ("Transaction");
      String symbol = (String) transaction.getProperty ("Symbol");
      Integer share = (Integer) transaction.getProperty ("Share");
      Boolean buy = (Boolean) transaction.getProperty ("Buy");
      SoapPrimitive price = (SoapPrimitive) transaction.getProperty ("Price");

      Date execTime = (Date) orderStatus.getProperty ("ExecTime");
      Calendar cal = Calendar.getInstance();
      cal.setTime (execTime);

      form.append (new StringItem ("Example #2 Pass #2", ""));
      form.append (new StringItem ("Customer Name", customerName));
      form.append (new StringItem ("Stock Symbol", symbol));
      form.append (new StringItem ("Share", share.toString()));
      form.append (new StringItem ("Buy?", buy.toString()));
      form.append (new StringItem ("Price", price.toString()));
      form.append (new StringItem ("ExecTime",
                     cal.get(Calendar.HOUR_OF_DAY) + ":" +
                     cal.get(Calendar.MINUTE) + ":" +
                     cal.get(Calendar.SECOND) + " " +
                     cal.get(Calendar.YEAR) + "-" +
                     cal.get(Calendar.MONTH) + "-" +
                     cal.get(Calendar.DATE)
                  ));
    } catch (Exception e) {
      e.printStackTrace ();
    }

    // Example #3 Pass #1
    try {
      SoapObject orderStatus = processRespMesg03 (arraySoapRespMesg);

      String customerName = (String) orderStatus.getProperty ("CustomerName");

      Vector transactions = (Vector) orderStatus.getProperty ("Transactions"); 

      // First element in the array
      SoapObject transaction0 = (SoapObject) transactions.elementAt(0);
      String symbol0 = (String) transaction0.getProperty ("Symbol");
      Integer share0 = (Integer) transaction0.getProperty ("Share");
      Boolean buy0 = (Boolean) transaction0.getProperty ("Buy");
      SoapPrimitive price0 = (SoapPrimitive) transaction0.getProperty ("Price");

      // Second element in the array
      SoapObject transaction1 = (SoapObject) transactions.elementAt(1);
      String symbol1 = (String) transaction1.getProperty ("Symbol");
      Integer share1 = (Integer) transaction1.getProperty ("Share");
      Boolean buy1 = (Boolean) transaction1.getProperty ("Buy");
      SoapPrimitive price1 = (SoapPrimitive) transaction1.getProperty ("Price");

      Date execTime = (Date) orderStatus.getProperty ("ExecTime");
      Calendar cal = Calendar.getInstance();
      cal.setTime (execTime);

      form.append (new StringItem ("Example #3 Pass #1", ""));
      form.append (new StringItem ("Customer Name", customerName));
      form.append (new StringItem ("Stock Symbol 0", symbol0));
      form.append (new StringItem ("Share 0", share0.toString()));
      form.append (new StringItem ("Buy? 0", buy0.toString()));
      form.append (new StringItem ("Price 0", price0.toString()));
      form.append (new StringItem ("Stock Symbol 1", symbol1));
      form.append (new StringItem ("Share 1", share1.toString()));
      form.append (new StringItem ("Buy? 1", buy1.toString()));
      form.append (new StringItem ("Price 1", price1.toString()));
      form.append (new StringItem ("ExecTime",
                     cal.get(Calendar.HOUR_OF_DAY) + ":" +
                     cal.get(Calendar.MINUTE) + ":" +
                     cal.get(Calendar.SECOND) + " " +
                     cal.get(Calendar.YEAR) + "-" +
                     cal.get(Calendar.MONTH) + "-" +
                     cal.get(Calendar.DATE)
                  ));
    } catch (Exception e) {
      e.printStackTrace ();
    }

    form.addCommand (doneCommand);
    form.setCommandListener ((CommandListener) this);
    display.setCurrent(form);
  }

  public void pauseApp() {
  }
 
  public void destroyApp(boolean unconditional) {
  }
 
  public void commandAction(Command command, Displayable screen) {
    if (command == doneCommand) {
      destroyApp(false);
      notifyDestroyed();
    } else {
      // Do nothing.
    }
  }
}
